@@ -39,7 +39,6 @@
 #include <netinet/in.h>
 #include <net/if.h>
 
-#include "bdev.h"
 #include "error.h"
 #include "commands.h"
 #include "list.h"
@@ -49,6 +48,7 @@
 #include "cgroup.h"
 #include "start.h"
 #include "state.h"
+#include "storage.h"
 
 #if IS_BIONIC
 #include <../include/lxcmntent.h>
@@ -165,7 +165,7 @@ static int cgroup_rmdir(char *dirname)
 
 	dir = opendir(dirname);
 	if (!dir) {
-		ERROR("%s: failed to open %s", __func__, dirname);
+		ERROR("Failed to open %s", dirname);
 		return -1;
 	}
 
@@ -190,7 +190,7 @@ static int cgroup_rmdir(char *dirname)
 		}
 		ret = lstat(pathname, &mystat);
 		if (ret) {
-			SYSERROR("%s: failed to stat %s", __func__, pathname);
+			SYSERROR("Failed to stat %s", pathname);
 			failed=1;
 			if (!saved_errno)
 				saved_errno = errno;
@@ -206,7 +206,7 @@ static int cgroup_rmdir(char *dirname)
 	}
 
 	if (rmdir(dirname) < 0) {
-		SYSERROR("%s: failed to delete %s", __func__, dirname);
+		SYSERROR("Failed to delete %s", dirname);
 		if (!saved_errno)
 			saved_errno = errno;
 		failed=1;
@@ -214,7 +214,7 @@ static int cgroup_rmdir(char *dirname)
 
 	ret = closedir(dir);
 	if (ret) {
-		SYSERROR("%s: failed to close directory %s", __func__, dirname);
+		SYSERROR("Failed to close directory %s", dirname);
 		if (!saved_errno)
 			saved_errno = errno;
 		failed=1;
@@ -791,7 +791,7 @@ static char *cgroup_rename_nsgroup(const char *mountpath, const char *oldname, p
 
 	len = strlen(oldname) + strlen(mountpath) + 22;
 	fulloldpath = alloca(len);
-	ret = snprintf(fulloldpath, len, "%s/%s/%ld", mountpath, oldname, (unsigned long)pid);
+	ret = snprintf(fulloldpath, len, "%s/%s/%lu", mountpath, oldname, (unsigned long)pid);
 	if (ret < 0 || ret >= len)
 		return NULL;
 
@@ -1874,7 +1874,8 @@ static int create_or_remove_cgroup(bool do_remove,
 			return 0;
 		if (recurse) {
 			if (conf && !lxc_list_empty(&conf->id_map))
-				r = userns_exec_1(conf, rmdir_wrapper, buf);
+				r = userns_exec_1(conf, rmdir_wrapper, buf,
+						  "rmdir_wrapper");
 			else
 				r = cgroup_rmdir(buf);
 		} else
@@ -2228,7 +2229,7 @@ static int cgroup_read_from_file(const char *fn, char buf[], size_t bufsize)
 			return ret;
 		}
 		/* Callers don't do this, but regression/sanity check */
-		ERROR("%s: was not expecting 0 bufsize", __func__);
+		ERROR("was not expecting 0 bufsize");
 		return -1;
 	}
 	buf[ret] = '\0';
@@ -2616,7 +2617,8 @@ static bool do_cgfs_chown(char *cgroup_path, struct lxc_conf *conf)
 	/* Unpriv users can't chown it themselves, so chown from
 	 * a child namespace mapping both our own and the target uid
 	 */
-	if (userns_exec_1(conf, chown_cgroup_wrapper, &data) < 0) {
+	if (userns_exec_1(conf, chown_cgroup_wrapper, &data,
+			  "chown_cgroup_wrapper") < 0) {
 		ERROR("Error requesting cgroup chown in new namespace");
 		return false;
 	}
