@@ -41,7 +41,6 @@
 #include <net/if.h>
 #include <poll.h>
 
-#include "bdev.h"
 #include "error.h"
 #include "commands.h"
 #include "list.h"
@@ -51,6 +50,7 @@
 #include "cgroup.h"
 #include "start.h"
 #include "state.h"
+#include "storage.h"
 
 #define CGM_SUPPORTS_GET_ABS 3
 #define CGM_SUPPORTS_NAMED 4
@@ -395,7 +395,7 @@ static int do_chown_cgroup(const char *controller, const char *cgroup_path,
 		goto out;
 	}
 	if (send_creds(sv[0], getpid(), getuid(), getgid())) {
-		SYSERROR("%s: Error sending pid over SCM_CREDENTIAL", __func__);
+		SYSERROR("Error sending pid over SCM_CREDENTIAL");
 		goto out;
 	}
 	fds.fd = sv[0];
@@ -410,7 +410,7 @@ static int do_chown_cgroup(const char *controller, const char *cgroup_path,
 		goto out;
 	}
 	if (send_creds(sv[0], getpid(), newuid, 0)) {
-		SYSERROR("%s: Error sending pid over SCM_CREDENTIAL", __func__);
+		SYSERROR("Error sending pid over SCM_CREDENTIAL");
 		goto out;
 	}
 	fds.fd = sv[0];
@@ -497,7 +497,8 @@ static bool chown_cgroup(const char *cgroup_path, struct lxc_conf *conf)
 	/* Unpriv users can't chown it themselves, so chown from
 	 * a child namespace mapping both our own and the target uid
 	 */
-	if (userns_exec_1(conf, chown_cgroup_wrapper, &data) < 0) {
+	if (userns_exec_1(conf, chown_cgroup_wrapper, &data,
+			  "chown_cgroup_wrapper") < 0) {
 		ERROR("Error requesting cgroup chown in new namespace");
 		return false;
 	}
