@@ -28,7 +28,6 @@
 #include <fcntl.h>
 #include <poll.h>
 #include <sys/socket.h>
-#include <inttypes.h>
 #include <sys/un.h>
 #include <sys/param.h>
 #include <malloc.h>
@@ -41,9 +40,12 @@
 #include "utils.h"
 #include "cgroup.h"
 #include "commands.h"
+#include "commands_utils.h"
 #include "console.h"
 #include "confile.h"
+#include "lxclock.h"
 #include "mainloop.h"
+#include "monitor.h"
 #include "af_unix.h"
 #include "config.h"
 
@@ -74,69 +76,20 @@
 
 lxc_log_define(lxc_commands, lxc);
 
-static int fill_sock_name(char *path, int len, const char *name,
-			  const char *lxcpath, const char *hashed_sock_name)
-{
-	char *tmppath;
-	size_t tmplen;
-	uint64_t hash;
-	int ret;
-
-	if (hashed_sock_name != NULL) {
-		ret = snprintf(path, len, "lxc/%s/command", hashed_sock_name);
-		if (ret < 0 || ret >= len) {
-			ERROR("Error writing to command sock path");
-			return -1;
-		}
-		return 0;
-	}
-
-	if (!lxcpath) {
-		lxcpath = lxc_global_config_value("lxc.lxcpath");
-		if (!lxcpath) {
-			ERROR("Out of memory getting lxcpath");
-			return -1;
-		}
-	}
-
-	ret = snprintf(path, len, "%s/%s/command", lxcpath, name);
-	if (ret < 0) {
-		ERROR("Error writing to command sock path");
-		return -1;
-	}
-	if (ret < len)
-		return 0;
-
-	/* ret >= len; lxcpath or name is too long.  hash both */
-	tmplen = strlen(name) + strlen(lxcpath) + 2;
-	tmppath = alloca(tmplen);
-	ret = snprintf(tmppath, tmplen, "%s/%s", lxcpath, name);
-	if (ret < 0 || ret >= tmplen) {
-		ERROR("memory error");
-		return -1;
-	}
-	hash = fnv_64a_buf(tmppath, ret, FNV1A_64_INIT);
-	ret = snprintf(path, len, "lxc/%016" PRIx64 "/command", hash);
-	if (ret < 0 || ret >= len) {
-		ERROR("Command socket name too long");
-		return -1;
-	}
-
-	return 0;
-}
-
 static const char *lxc_cmd_str(lxc_cmd_t cmd)
 {
 	static const char * const cmdname[LXC_CMD_MAX] = {
-		[LXC_CMD_CONSOLE]         = "console",
-		[LXC_CMD_STOP]            = "stop",
-		[LXC_CMD_GET_STATE]       = "get_state",
-		[LXC_CMD_GET_INIT_PID]    = "get_init_pid",
-		[LXC_CMD_GET_CLONE_FLAGS] = "get_clone_flags",
-		[LXC_CMD_GET_CGROUP]      = "get_cgroup",
-		[LXC_CMD_GET_CONFIG_ITEM] = "get_config_item",
-		[LXC_CMD_GET_NAME]        = "get_name",
-		[LXC_CMD_GET_LXCPATH]     = "get_lxcpath",
+		[LXC_CMD_CONSOLE]          = "console",
+		[LXC_CMD_CONSOLE_WINCH]    = "console_winch",
+		[LXC_CMD_STOP]             = "stop",
+		[LXC_CMD_GET_STATE]        = "get_state",
+		[LXC_CMD_GET_INIT_PID]     = "get_init_pid",
+		[LXC_CMD_GET_CLONE_FLAGS]  = "get_clone_flags",
+		[LXC_CMD_GET_CGROUP]       = "get_cgroup",
+		[LXC_CMD_GET_CONFIG_ITEM]  = "get_config_item",
+		[LXC_CMD_GET_NAME]         = "get_name",
+		[LXC_CMD_GET_LXCPATH]      = "get_lxcpath",
+		[LXC_CMD_ADD_STATE_CLIENT] = "add_state_client",
 	};
 
 	if (cmd >= LXC_CMD_MAX)
@@ -163,15 +116,16 @@ static const char *lxc_cmd_str(lxc_cmd_t cmd)
  */
 static int lxc_cmd_rsp_recv(int sock, struct lxc_cmd_rr *cmd)
 {
-	int ret,rspfd;
+	int ret, rspfd;
 	struct lxc_cmd_rsp *rsp = &cmd->rsp;
 
-	ret = lxc_abstract_unix_recv_fd(sock, &rspfd, rsp, sizeof(*rsp));
+	ret = lxc_abstract_unix_recv_fds(sock, &rspfd, 1, rsp, sizeof(*rsp));
 	if (ret < 0) {
 		WARN("Command %s failed to receive response: %s.",
 		     lxc_cmd_str(cmd->req.cmd), strerror(errno));
 		return -1;
 	}
+	TRACE("Command \"%s received response", lxc_cmd_str(cmd->req.cmd));
 
 	if (cmd->req.cmd == LXC_CMD_CONSOLE) {
 		struct lxc_cmd_console_rsp_data *rspdata;
@@ -193,8 +147,11 @@ static int lxc_cmd_rsp_recv(int sock, struct lxc_cmd_rr *cmd)
 		rsp->data = rspdata;
 	}
 
-	if (rsp->datalen == 0)
+	if (rsp->datalen == 0) {
+		DEBUG("command %s response data length is 0",
+		      lxc_cmd_str(cmd->req.cmd));
 		return ret;
+	}
 	if (rsp->datalen > LXC_CMD_DATA_MAX) {
 		ERROR("Command %s response data %d too long.",
 		      lxc_cmd_str(cmd->req.cmd), rsp->datalen);
@@ -249,12 +206,61 @@ static int lxc_cmd_rsp_send(int fd, struct lxc_cmd_rsp *rsp)
 	return 0;
 }
 
+static int lxc_cmd_send(const char *name, struct lxc_cmd_rr *cmd,
+		 const char *lxcpath, const char *hashed_sock_name)
+{
+	int client_fd;
+	int ret = -1;
+
+	client_fd = lxc_cmd_connect(name, lxcpath, hashed_sock_name);
+	if (client_fd < 0 && client_fd == -ECONNREFUSED)
+		return -ECONNREFUSED;
+	else if (client_fd < 0)
+		return -1;
+
+	TRACE("Command \"%s\" connected to command socket",
+	      lxc_cmd_str(cmd->req.cmd));
+
+	ret = lxc_abstract_unix_send_credential(client_fd, &cmd->req, sizeof(cmd->req));
+	if (ret != sizeof(cmd->req)) {
+		close(client_fd);
+
+		if (errno == EPIPE)
+			return -EPIPE;
+
+		if (ret >= 0)
+			return -EMSGSIZE;
+
+		return -1;
+	}
+
+	TRACE("Command \"%s\" requested data of length %d",
+	      lxc_cmd_str(cmd->req.cmd), cmd->req.datalen);
+
+	if (cmd->req.datalen > 0) {
+		ret = send(client_fd, cmd->req.data, cmd->req.datalen, MSG_NOSIGNAL);
+		if (ret != cmd->req.datalen) {
+			close(client_fd);
+
+			if (errno == EPIPE)
+				return -EPIPE;
+
+			if (ret >= 0)
+				return -EMSGSIZE;
+
+			return -1;
+		}
+	}
+
+	return client_fd;
+}
+
 /*
  * lxc_cmd: Connect to the specified running container, send it a command
  * request and collect the response
  *
  * @name           : name of container to connect to
- * @cmd            : command with initialized reqest to send
+ * @cmd            : command with initialized request to send
  * @stopped        : output indicator if the container was not running
  * @lxcpath        : the lxcpath in which the container is running
  *
@@ -271,68 +277,43 @@ static int lxc_cmd_rsp_send(int fd, struct lxc_cmd_rsp *rsp)
 static int lxc_cmd(const char *name, struct lxc_cmd_rr *cmd, int *stopped,
 		   const char *lxcpath, const char *hashed_sock_name)
 {
-	int sock, ret = -1;
-	char path[sizeof(((struct sockaddr_un *)0)->sun_path)] = { 0 };
-	char *offset = &path[1];
-	int len;
-	int stay_connected = cmd->req.cmd == LXC_CMD_CONSOLE;
+	int client_fd, ret = -1;
+	bool stay_connected = false;
+
+	if (cmd->req.cmd == LXC_CMD_CONSOLE ||
+	    cmd->req.cmd == LXC_CMD_ADD_STATE_CLIENT)
+		stay_connected = true;
 
 	*stopped = 0;
 
-	/* -2 here because this is an abstract unix socket so it needs a
-	 * leading \0, and we null terminate, so it needs a trailing \0.
-	 * Although null termination isn't required by the API, we do it anyway
-	 * because we print the sockname out sometimes.
-	 */
-	len = sizeof(path)-2;
-	if (fill_sock_name(offset, len, name, lxcpath, hashed_sock_name))
-		return -1;
+	TRACE("command %s tries to connect command socket",
+	      lxc_cmd_str(cmd->req.cmd));
 
-	sock = lxc_abstract_unix_connect(path);
-	if (sock < 0) {
-		if (errno == ECONNREFUSED)
+	client_fd = lxc_cmd_send(name, cmd, lxcpath, hashed_sock_name);
+	if (client_fd < 0) {
+		TRACE("command %s failed to connect command socket: %s",
+		      lxc_cmd_str(cmd->req.cmd), strerror(errno));
+		if (client_fd == -ECONNREFUSED) {
 			*stopped = 1;
-		else
-			SYSERROR("Command %s failed to connect to \"@%s\".",
-				 lxc_cmd_str(cmd->req.cmd), offset);
-		return -1;
-	}
+			return -1;
+		}
 
-	ret = lxc_abstract_unix_send_credential(sock, &cmd->req, sizeof(cmd->req));
-	if (ret != sizeof(cmd->req)) {
-		if (errno == EPIPE)
+		if (client_fd == -EPIPE)
 			goto epipe;
-		SYSERROR("Command %s failed to send req to \"@%s\" %d.",
-			 lxc_cmd_str(cmd->req.cmd), offset, ret);
-		if (ret >=0)
-			ret = -1;
+
 		goto out;
 	}
 
-	if (cmd->req.datalen > 0) {
-		ret = send(sock, cmd->req.data, cmd->req.datalen, MSG_NOSIGNAL);
-		if (ret != cmd->req.datalen) {
-			if (errno == EPIPE)
-				goto epipe;
-			SYSERROR("Command %s failed to send request data to \"@%s\" %d.",
-				 lxc_cmd_str(cmd->req.cmd), offset, ret);
-			if (ret >=0)
-				ret = -1;
-			goto out;
-		}
-	}
-
-	ret = lxc_cmd_rsp_recv(sock, cmd);
+	ret = lxc_cmd_rsp_recv(client_fd, cmd);
 out:
 	if (!stay_connected || ret <= 0)
-		close(sock);
+		close(client_fd);
 	if (stay_connected && ret > 0)
-		cmd->rsp.ret = sock;
+		cmd->rsp.ret = client_fd;
 
 	return ret;
 
 epipe:
-	close(sock);
 	*stopped = 1;
 	return 0;
 }
@@ -454,20 +435,26 @@ char *lxc_cmd_get_cgroup_path(const char *name, const char *lxcpath,
 	};
 
 	ret = lxc_cmd(name, &cmd, &stopped, lxcpath, NULL);
-	if (ret < 0)
+	if (ret < 0) {
+		TRACE("command %s failed for container \"%s\": %s.",
+		      lxc_cmd_str(cmd.req.cmd), name, strerror(errno));
 		return NULL;
+	}
 
 	if (!ret) {
-		WARN("Container \"%s\" has stopped before sending its state.", name);
+		WARN("container \"%s\" has stopped before sending its state", name);
 		return NULL;
 	}
 
 	if (cmd.rsp.ret < 0 || cmd.rsp.datalen < 0) {
-		ERROR("Command %s failed for container \"%s\": %s.",
+		ERROR("command %s failed for container \"%s\": %s",
 		      lxc_cmd_str(cmd.req.cmd), name, strerror(-cmd.rsp.ret));
 		return NULL;
 	}
 
+	TRACE("command %s successful for container \"%s\"",
+	      lxc_cmd_str(cmd.req.cmd), name);
+
 	return cmd.rsp.data;
 }
 
@@ -526,14 +513,18 @@ static int lxc_cmd_get_config_item_callback(int fd, struct lxc_cmd_req *req,
 	int cilen;
 	struct lxc_cmd_rsp rsp;
 	char *cidata;
+	struct lxc_config_t *item;
 
 	memset(&rsp, 0, sizeof(rsp));
-	cilen = lxc_get_config_item(handler->conf, req->data, NULL, 0);
+	item = lxc_getconfig(req->data);
+	if (!item)
+		goto err1;
+	cilen = item->get(req->data, NULL, 0, handler->conf);
 	if (cilen <= 0)
 		goto err1;
 
 	cidata = alloca(cilen + 1);
-	if (lxc_get_config_item(handler->conf, req->data, cidata, cilen + 1) != cilen)
+	if (item->get(req->data, cidata, cilen + 1, handler->conf) != cilen)
 		goto err1;
 	cidata[cilen] = '\0';
 	rsp.data = cidata;
@@ -555,7 +546,7 @@ out:
  *
  * Returns the state on success, < 0 on failure
  */
-lxc_state_t lxc_cmd_get_state(const char *name, const char *lxcpath)
+int lxc_cmd_get_state(const char *name, const char *lxcpath)
 {
 	int ret, stopped;
 	struct lxc_cmd_rr cmd = {
@@ -744,7 +735,7 @@ static int lxc_cmd_console_callback(int fd, struct lxc_cmd_req *req,
 
 	memset(&rsp, 0, sizeof(rsp));
 	rsp.data = INT_TO_PTR(ttynum);
-	if (lxc_abstract_unix_send_fd(fd, masterfd, &rsp, sizeof(rsp)) < 0) {
+	if (lxc_abstract_unix_send_fds(fd, &masterfd, 1, &rsp, sizeof(rsp)) < 0) {
 		ERROR("Failed to send tty to client.");
 		lxc_console_free(handler->conf, fd);
 		goto out_close;
@@ -835,22 +826,129 @@ static int lxc_cmd_get_lxcpath_callback(int fd, struct lxc_cmd_req *req,
 	return lxc_cmd_rsp_send(fd, &rsp);
 }
 
+int lxc_cmd_add_state_client(const char *name, const char *lxcpath,
+			     lxc_state_t states[MAX_STATE],
+			     int *state_client_fd)
+{
+	int stopped;
+	ssize_t ret;
+	int state = -1;
+	struct lxc_cmd_rr cmd = {
+	    .req = {
+		.cmd     = LXC_CMD_ADD_STATE_CLIENT,
+		.data    = states,
+		.datalen = (sizeof(lxc_state_t) * MAX_STATE)
+	    },
+	};
+
+	/* Lock the whole lxc_cmd_add_state_client_callback() call to ensure
+	*  that lxc_set_state() doesn't cause us to miss a state.
+	 */
+	process_lock();
+	/* Check if already in requested state. */
+	state = lxc_getstate(name, lxcpath);
+	if (state < 0) {
+		process_unlock();
+		TRACE("failed to retrieve state of container: %s",
+		      strerror(errno));
+		return -1;
+	} else if (states[state]) {
+		process_unlock();
+		TRACE("container is %s state", lxc_state2str(state));
+		return state;
+	}
+
+	if ((state == STARTING) && !states[RUNNING] && !states[STOPPING] && !states[STOPPED]) {
+		process_unlock();
+		TRACE("container is in %s state and caller requested to be "
+		      "informed about a previous state",
+		      lxc_state2str(state));
+		return state;
+	} else if ((state == RUNNING) && !states[STOPPING] && !states[STOPPED]) {
+		process_unlock();
+		TRACE("container is in %s state and caller requested to be "
+		      "informed about a previous state",
+		      lxc_state2str(state));
+		return state;
+	} else if ((state == STOPPING) && !states[STOPPED]) {
+		process_unlock();
+		TRACE("container is in %s state and caller requested to be "
+		      "informed about a previous state",
+		      lxc_state2str(state));
+		return state;
+	} else if ((state == STOPPED) || (state == ABORTING)) {
+		process_unlock();
+		TRACE("container is in %s state and caller requested to be "
+		      "informed about a previous state",
+		      lxc_state2str(state));
+		return state;
+	}
+
+	ret = lxc_cmd(name, &cmd, &stopped, lxcpath, NULL);
+	process_unlock();
+	if (ret < 0) {
+		ERROR("failed to execute command: %s", strerror(errno));
+		return -1;
+	}
+	/* We should now be guaranteed to get an answer from the state sending
+	 * function.
+	 */
+
+	if (cmd.rsp.ret < 0) {
+		ERROR("failed to receive socket fd");
+		return -1;
+	}
+
+	*state_client_fd = cmd.rsp.ret;
+	return MAX_STATE;
+}
+
+static int lxc_cmd_add_state_client_callback(int fd, struct lxc_cmd_req *req,
+					     struct lxc_handler *handler)
+{
+	struct lxc_cmd_rsp rsp = {0};
+
+	if (req->datalen < 0) {
+		TRACE("Requested datalen was < 0");
+		return -1;
+	}
+
+	if (req->datalen > (sizeof(lxc_state_t) * MAX_STATE)) {
+		TRACE("Requested datalen was too large");
+		return -1;
+	}
+
+	if (!req->data) {
+		TRACE("No states requested");
+		return -1;
+	}
+
+	rsp.ret = lxc_add_state_client(fd, handler, (lxc_state_t *)req->data);
+	if (rsp.ret < 0)
+		ERROR("Failed to add state client %d to state client list", fd);
+	else
+		TRACE("Added state client %d to state client list", fd);
+
+	return lxc_cmd_rsp_send(fd, &rsp);
+}
+
 static int lxc_cmd_process(int fd, struct lxc_cmd_req *req,
 			   struct lxc_handler *handler)
 {
 	typedef int (*callback)(int, struct lxc_cmd_req *, struct lxc_handler *);
 
 	callback cb[LXC_CMD_MAX] = {
-		[LXC_CMD_CONSOLE]         = lxc_cmd_console_callback,
-		[LXC_CMD_CONSOLE_WINCH]   = lxc_cmd_console_winch_callback,
-		[LXC_CMD_STOP]            = lxc_cmd_stop_callback,
-		[LXC_CMD_GET_STATE]       = lxc_cmd_get_state_callback,
-		[LXC_CMD_GET_INIT_PID]    = lxc_cmd_get_init_pid_callback,
-		[LXC_CMD_GET_CLONE_FLAGS] = lxc_cmd_get_clone_flags_callback,
-		[LXC_CMD_GET_CGROUP]      = lxc_cmd_get_cgroup_callback,
-		[LXC_CMD_GET_CONFIG_ITEM] = lxc_cmd_get_config_item_callback,
-		[LXC_CMD_GET_NAME]        = lxc_cmd_get_name_callback,
-		[LXC_CMD_GET_LXCPATH]     = lxc_cmd_get_lxcpath_callback,
+		[LXC_CMD_CONSOLE]          = lxc_cmd_console_callback,
+		[LXC_CMD_CONSOLE_WINCH]    = lxc_cmd_console_winch_callback,
+		[LXC_CMD_STOP]             = lxc_cmd_stop_callback,
+		[LXC_CMD_GET_STATE]        = lxc_cmd_get_state_callback,
+		[LXC_CMD_GET_INIT_PID]     = lxc_cmd_get_init_pid_callback,
+		[LXC_CMD_GET_CLONE_FLAGS]  = lxc_cmd_get_clone_flags_callback,
+		[LXC_CMD_GET_CGROUP]       = lxc_cmd_get_cgroup_callback,
+		[LXC_CMD_GET_CONFIG_ITEM]  = lxc_cmd_get_config_item_callback,
+		[LXC_CMD_GET_NAME]         = lxc_cmd_get_name_callback,
+		[LXC_CMD_GET_LXCPATH]      = lxc_cmd_get_lxcpath_callback,
+		[LXC_CMD_ADD_STATE_CLIENT] = lxc_cmd_add_state_client_callback,
 	};
 
 	if (req->cmd >= LXC_CMD_MAX) {
@@ -884,24 +982,30 @@ static int lxc_cmd_handler(int fd, uint32_t events, void *data,
 		goto out_close;
 	}
 
+	TRACE("Processing \"%s\" command", lxc_cmd_str(req.cmd));
 	if (ret < 0) {
-		SYSERROR("Failed to receive data on command socket.");
+		SYSERROR("Failed to receive data on command socket for \"%s\"",
+			 lxc_cmd_str(req.cmd));
 		goto out_close;
 	}
 
 	if (!ret) {
-		DEBUG("Peer has disconnected.");
+		DEBUG("Peer has disconnected for \"%s\"", lxc_cmd_str(req.cmd));
 		goto out_close;
 	}
 
 	if (ret != sizeof(req)) {
-		WARN("Failed to receive full command request. Ignoring request.");
+		WARN("Failed to receive full command request. Ignoring request "
+		     "for \"%s\"",
+		     lxc_cmd_str(req.cmd));
 		ret = -1;
 		goto out_close;
 	}
 
 	if (req.datalen > LXC_CMD_DATA_MAX) {
-		ERROR("Received command data length %d is too large.", req.datalen);
+		ERROR("Received command data length %d is too large for "
+		      "command \"%s\"",
+		      req.datalen, lxc_cmd_str(req.cmd));
 		ret = -1;
 		goto out_close;
 	}
@@ -912,7 +1016,9 @@ static int lxc_cmd_handler(int fd, uint32_t events, void *data,
 		reqdata = alloca(req.datalen);
 		ret = recv(fd, reqdata, req.datalen, 0);
 		if (ret != req.datalen) {
-			WARN("Failed to receive full command request. Ignoring request.");
+			WARN("Failed to receive full command request. Ignoring "
+			     "request for \"%s\"",
+			     lxc_cmd_str(req.cmd));
 			ret = -1;
 			goto out_close;
 		}
@@ -982,8 +1088,9 @@ int lxc_cmd_init(const char *name, struct lxc_handler *handler,
 	 * Although null termination isn't required by the API, we do it anyway
 	 * because we print the sockname out sometimes.
 	 */
-	len = sizeof(path)-2;
-	if (fill_sock_name(offset, len, name, lxcpath, NULL))
+	len = sizeof(path) - 2;
+	if (lxc_make_abstract_socket_name(offset, len, name, lxcpath, NULL,
+					  "command"))
 		return -1;
 
 	fd = lxc_abstract_unix_open(path, SOCK_STREAM, 0);
