@@ -41,6 +41,7 @@
 #include "log.h"
 #include "caps.h"
 #include "utils.h"
+#include "lxccontainer.h"
 
 /* We're logging in seconds and nanoseconds. Assuming that the underlying
  * datatype is currently at maximum a 64bit integer, we have a date string that
@@ -62,7 +63,7 @@ lxc_log_define(lxc_log, lxc);
 static int log_append_stderr(const struct lxc_log_appender *appender,
 			     struct lxc_log_event *event)
 {
-	if (event->priority < LXC_LOG_PRIORITY_ERROR)
+	if (event->priority < LXC_LOG_LEVEL_ERROR)
 		return 0;
 
 	fprintf(stderr, "%s: ", log_prefix);
@@ -154,7 +155,7 @@ int lxc_unix_epoch_to_utc(char *buf, size_t bufsize, const struct timespec *time
 	seconds = (time->tv_sec - d_in_s - h_in_s - (minutes * 60));
 
 	/* Make string from nanoseconds. */
-	ret = snprintf(nanosec, LXC_NUMSTRLEN64, "%ld", time->tv_nsec);
+	ret = snprintf(nanosec, LXC_NUMSTRLEN64, "%"PRId64, (int64_t)time->tv_nsec);
 	if (ret < 0 || ret >= LXC_NUMSTRLEN64)
 		return -1;
 
@@ -227,7 +228,8 @@ static int log_append_logfile(const struct lxc_log_appender *appender,
 
 	if ((size_t)n < (sizeof(buffer) - 1))
 		n += vsnprintf(buffer + n, sizeof(buffer) - n, event->fmt, *event->vap);
-	else
+
+	if ((size_t)n >= sizeof(buffer))
 		n = sizeof(buffer) - 1;
 
 	buffer[n] = '\n';
@@ -249,14 +251,14 @@ static struct lxc_log_appender log_appender_logfile = {
 
 static struct lxc_log_category log_root = {
 	.name		= "root",
-	.priority	= LXC_LOG_PRIORITY_ERROR,
+	.priority	= LXC_LOG_LEVEL_ERROR,
 	.appender	= NULL,
 	.parent		= NULL,
 };
 
 struct lxc_log_category lxc_log_category_lxc = {
 	.name		= "lxc",
-	.priority	= LXC_LOG_PRIORITY_ERROR,
+	.priority	= LXC_LOG_LEVEL_ERROR,
 	.appender	= &log_appender_logfile,
 	.parent		= &log_root
 };
@@ -443,11 +445,9 @@ static int _lxc_log_set_file(const char *name, const char *lxcpath, int create_d
  * Called from lxc front-end programs (like lxc-create, lxc-start) to
  * initalize the log defaults.
  */
-extern int lxc_log_init(const char *name, const char *file,
-			const char *priority, const char *prefix, int quiet,
-			const char *lxcpath)
+extern int lxc_log_init(struct lxc_log *log)
 {
-	int lxc_priority = LXC_LOG_PRIORITY_ERROR;
+	int lxc_priority = LXC_LOG_LEVEL_ERROR;
 	int ret;
 
 	if (lxc_log_fd != -1) {
@@ -455,8 +455,8 @@ extern int lxc_log_init(const char *name, const char *file,
 		return 0;
 	}
 
-	if (priority)
-		lxc_priority = lxc_log_priority_to_int(priority);
+	if (log->level)
+		lxc_priority = lxc_log_priority_to_int(log->level);
 
 	if (!lxc_loglevel_specified) {
 		lxc_log_category_lxc.priority = lxc_priority;
@@ -464,46 +464,46 @@ extern int lxc_log_init(const char *name, const char *file,
 	}
 
 	if (!lxc_quiet_specified) {
-		if (!quiet)
+		if (!log->quiet)
 			lxc_log_category_lxc.appender->next = &log_appender_stderr;
 	}
 
-	if (prefix)
-		lxc_log_set_prefix(prefix);
+	if (log->prefix)
+		lxc_log_set_prefix(log->prefix);
 
-	if (file) {
-		if (strcmp(file, "none") == 0)
+	if (log->file) {
+		if (strcmp(log->file, "none") == 0)
 			return 0;
-		ret = __lxc_log_set_file(file, 1);
+		ret = __lxc_log_set_file(log->file, 1);
 		lxc_log_use_global_fd = 1;
 	} else {
 		/* if no name was specified, there nothing to do */
-		if (!name)
+		if (!log->name)
 			return 0;
 
 		ret = -1;
 
-		if (!lxcpath)
-			lxcpath = LOGPATH;
+		if (!log->lxcpath)
+			log->lxcpath = LOGPATH;
 
 		/* try LOGPATH if lxcpath is the default for the privileged containers */
-		if (!geteuid() && strcmp(LXCPATH, lxcpath) == 0)
-			ret = _lxc_log_set_file(name, NULL, 0);
+		if (!geteuid() && strcmp(LXCPATH, log->lxcpath) == 0)
+			ret = _lxc_log_set_file(log->name, NULL, 0);
 
 		/* try in lxcpath */
 		if (ret < 0)
-			ret = _lxc_log_set_file(name, lxcpath, 1);
+			ret = _lxc_log_set_file(log->name, log->lxcpath, 1);
 
 		/* try LOGPATH in case its writable by the caller */
 		if (ret < 0)
-			ret = _lxc_log_set_file(name, NULL, 0);
+			ret = _lxc_log_set_file(log->name, NULL, 0);
 	}
 
 	/*
 	 * If !file, that is, if the user did not request this logpath, then
 	 * ignore failures and continue logging to console
 	 */
-	if (!file && ret != 0) {
+	if (!log->file && ret != 0) {
 		INFO("Ignoring failure to open default logfile.");
 		ret = 0;
 	}
@@ -518,7 +518,7 @@ extern int lxc_log_init(const char *name, const char *file,
  */
 extern int lxc_log_set_level(int *dest, int level)
 {
-	if (level < 0 || level >= LXC_LOG_PRIORITY_NOTSET) {
+	if (level < 0 || level >= LXC_LOG_LEVEL_NOTSET) {
 		ERROR("invalid log priority %d", level);
 		return -1;
 	}
@@ -534,7 +534,7 @@ extern int lxc_log_get_level(void)
 extern bool lxc_log_has_valid_level(void)
 {
 	int log_level = lxc_log_get_level();
-	if (log_level < 0 || log_level >= LXC_LOG_PRIORITY_NOTSET)
+	if (log_level < 0 || log_level >= LXC_LOG_LEVEL_NOTSET)
 		return false;
 	return true;
 }
