@@ -37,13 +37,13 @@
 #include <lxc/lxccontainer.h>
 
 #include "attach.h"
-#include "bdev.h"
 #include "log.h"
 #include "confile.h"
 #include "arguments.h"
 #include "lxc.h"
 #include "conf.h"
 #include "state.h"
+#include "storage.h"
 #include "utils.h"
 
 #ifndef HAVE_GETSUBOPT
@@ -158,6 +158,7 @@ static int parse_ovl_mnt(char *mntstring, enum mnttype type);
 int main(int argc, char *argv[])
 {
 	struct lxc_container *c;
+	struct lxc_log log;
 	int flags = 0;
 	int ret = EXIT_FAILURE;
 
@@ -167,8 +168,14 @@ int main(int argc, char *argv[])
 	if (!my_args.log_file)
 		my_args.log_file = "none";
 
-	if (lxc_log_init(my_args.name, my_args.log_file, my_args.log_priority,
-			 my_args.progname, my_args.quiet, my_args.lxcpath[0]))
+	log.name = my_args.name;
+	log.file = my_args.log_file;
+	log.level = my_args.log_priority;
+	log.prefix = my_args.progname;
+	log.quiet = my_args.quiet;
+	log.lxcpath = my_args.lxcpath[0];
+
+	if (lxc_log_init(&log))
 		exit(ret);
 	lxc_log_options_no_override();
 
